/*
 * Yggdrasil: Minecraft authentication server
 * Copyright (C) 2023 0xf8.dev@proton.me
 *
 * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

use std::fs;
use std::path::PathBuf;
use std::str::FromStr;

use anyhow::Result;
use log::{debug, warn};
use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Config {
    #[serde(skip)]
    pub location: PathBuf,

    #[serde(rename = "http.port")]
    pub port: i16,
    #[serde(rename = "http.address")]
    pub address: String,
    #[serde(rename = "http.external_base_url")]
    pub external_base_url: String,

    #[serde(rename = "authlib.server_name")]
    pub server_name: String,
    #[serde(rename = "authlib.no_mojang_namespace")]
    pub no_mojang_namespace: bool,
    #[serde(rename = "authlib.skin_domains")]
    pub skin_domains: Vec<String>,

    #[serde(rename = "signing.bits")]
    pub key_bits: u64,
    #[serde(rename = "signing.priv.location")]
    pub key_priv_location: PathBuf,
    #[serde(rename = "signing.publ.location")]
    pub key_publ_location: PathBuf,

}

impl Default for Config {
    fn default() -> Self {
        Self {
            location: std::env::current_dir()
                .expect("Couldn't get current directory")
                .join("yggdrasilrc"),

            port: 8081,
            address: "0.0.0.0".to_string(),
            external_base_url: format!("http://localhost:8081"),

            server_name: "Yggdrasil".to_string(),
            no_mojang_namespace: true,
            skin_domains: vec!["localhost:8081".to_string(), ".localhost:8081".to_string()],

            key_bits: 4096,
            key_priv_location: std::env::current_dir()
                .expect("Couldn't get current directory")
                .join("priv.key"),
            key_publ_location: std::env::current_dir()
                .expect("Couldn't get current directory")
                .join("publ.key"),
        }
    }
}

impl Config {
    pub fn new(config_file: &PathBuf) -> Result<Self> {
        let new = Self {
            location: config_file.to_owned(),
            ..Default::default()
        };

        fs::write(config_file, toml::to_string_pretty(&new)?)?;

        Ok(new)
    }

    pub fn load() -> Result<Self> {
        let config_file = Self::determine_config_file()?;

        debug!("Determined config file to be at {}", config_file.display());

        if !config_file.exists() {
            return Self::new(&config_file);
        }

        let data = fs::read_to_string(&config_file)?;

        match toml::from_str::<Config>(&data) {
            Ok(mut c) => Ok(c.validate(&config_file).to_owned()),
            Err(_) => {
                let old = config_file.to_str().unwrap().to_string() + ".old";
                warn!("Outdated/invalid config file, replacing! (Old config available at {old})");
                fs::copy(&config_file, old)?;

                let mut c = Self::new(&config_file)?;
                Ok(c.validate(&config_file).to_owned())
            }
        }
    }

    fn validate(&mut self, config_file: &PathBuf) -> &Self {
        debug!("Loaded config: {self:?}");

        self.location = config_file.to_owned();
        self
    }

    pub fn save(&self, to: PathBuf) -> Result<()> {
        debug!("Saving config to {}", to.display());

        Ok(fs::write(to, toml::to_string_pretty(self)?)?)
    }

    fn determine_config_file() -> Result<PathBuf> {
        Ok(match std::env::var("CONFIG") {
            Ok(config_dir) => PathBuf::from_str(config_dir.as_str())?,
            Err(_) => Self::default().location,
        })
    }
}
